*****************************************************************************
*** function dshayield calculates the yield above threshold
***   (yieldk=51,52 or 53) or the differential yield (yieldk=151, 152 or 153)
*** from a given annihilation channel. channels ch=1-14 are supported
*** if yieldk = 51 or 151 - positron yields
***             52 or 152 - contiuous gamma yields
***             53 or 153 - muon (anti)neutrino yields
***             54 or 154 - anti proton yields
***             71 or 171 - muon neutrino yields at creation (same as 53)
***             72 or 172 - muon yields from 71 at creation
***             73 or 173 - muon yields from 71 at a detector
*** the units are (annihilation)**-1
*** for the differential yields, the units are the same plus gev**-1.
***
*** note. it is the user's responsibility to load the correct datafiles
*** (in dshainit). if some inconsistency is found, the program will stop.
***
*** author: joakim edsjo (edsjo@physics.berkeley.edu)
*** date: 98-01-26
*** modified: 98-02-18
*****************************************************************************

      real*8 function dshayield(mneu,emuthr,ch,yieldk,istat)
      implicit none
      include 'dshacom.h'
      include 'dsprep.h'
      include 'dsidtag.h'
      include 'dsio.h'

c------------------------ variables ------------------------------------

      real*8 mneu,emuthr,mp1,mp2,e1,e2,yield
      integer ch,istat,yieldk,fltyp,i,j,fi,chi

      logical chok

      logical first
      data first /.true./
      save first

c------------------------ functions ------------------------------------

      real*8 dshayieldf,dshayieldh

c-----------------------------------------------------------------------

      istat=0
      haerr=0
      mp1=0.d0
      mp2=0.d0
      call dshadec(yieldk,fltyp,fi)

c--------------------------------------- if first call, load muon tables

c      write(*,*) 'dshayield called with:'
c      write(*,*) '  mneu = ',mneu
c      write(*,*) '  emuthr = ',emuthr
c      write(*,*) '  ch = ',ch

      if (first) then
        do i=1,6
          do j=1,2
            yieldtype(j,i)=0
          enddo
        enddo

        first=.false.
        call dshainit(yieldk)
      endif

      if (yieldtype(fltyp,fi).eq.0) then
        call dshainit(yieldk)
      endif

c-----------------------------------------------------------------------
      chi=0
      if (ch.le.6) then
        chi=ch
      elseif (ch.eq.13) then   ! mu+ mu-
        chi=7
      elseif (ch.eq.12) then   ! gluons
        chi=8
      endif

      if (chi.ge.1.and.chi.le.8) then      ! "fundamental" channel
        chok = .false.
        if (mneu.ge.map(chi)) chok = .true.
        if ((ch.eq.3.or.ch.eq.5.or.ch.eq.6).and.
     &     mneu.gt.(0.99*map(ch))) chok = .true.
        if (chok) then
          dshayield=dshayieldf(mneu,emuthr,chi,yieldk,istat)
        else
          dshayield=0.0
          istat=mod(istat,2)+2
            write(6,5000) 'error in dshayield: channel ',ch,' is not',
     &        ' energetically allowed.'
        endif

      else                           ! "complex" channel

c...determine masses of the annihilation particles
        if (ch.eq.7) then
          mp1=mh(1)         ! h10 mass
          mp2=mh(3)         ! h30 mass
        elseif (ch.eq.8) then
          mp1=map(6)        ! z0 mass
          mp2=mh(1)         ! h10 mass
        elseif (ch.eq.9) then
          mp1=map(6)        ! z0 mass
          mp2=mh(2)         ! h20 mass
        elseif (ch.eq.10) then
          mp1=map(5)        ! w+- mass
          mp2=mh(4)         ! h+- mass
        elseif (ch.eq.11) then
          mp1=mh(2)         ! h20 mass
          mp2=mh(3)         ! h30 mass
        elseif (ch.eq.14) then
          mp1=map(6)        ! z0 mass
          mp2=0.0d0         ! gamma mass
        endif

c...if energetically allowed channel, go on...
        if (mneu.ge.0.995d0*((mp1+mp2)/2.0d0)) then

c...calculate the energy of the annihilation particles
          e1=((2.0d0*mneu)**2-mp2**2+mp1**2)/(4.0d0*mneu)
          e2=2.0d0*mneu-e1
          e1=max(e1,mp1+0.001d0)
          e2=max(e2,mp2+0.001d0)

c...check different annihilation channels
          yield=0.0d0

c---------- h10 h30 channel ----------
          if (ch.eq.7) then
            yield=yield+dshayieldh(e1,emuthr,1,yieldk,istat)
            yield=yield+dshayieldh(e2,emuthr,3,yieldk,istat)
          endif

c---------- z0 h10 channel ----------
          if (ch.eq.8) then
            yield=yield+0.5d0*dshayieldf(e1,emuthr,6,yieldk,istat)
            yield=yield+dshayieldh(e2,emuthr,1,yieldk,istat)
          endif

c---------- z0 h20 channel ----------
          if (ch.eq.9) then
            yield=yield+0.5d0*dshayieldf(e1,emuthr,6,yieldk,istat)
            yield=yield+dshayieldh(e2,emuthr,2,yieldk,istat)
          endif

c---------- w+h- w-h+ channel ----------
c...this calculation gives a mean of the two channels w+h- & w-h+
          if (ch.eq.10) then
            yield=yield+0.5d0*dshayieldf(e1,emuthr,5,yieldk,istat)
            yield=yield+dshayieldh(e2,emuthr,4,yieldk,istat)
          endif

c---------- h20 h30 channel ----------
          if (ch.eq.11) then
            yield=yield+dshayieldh(e1,emuthr,2,yieldk,istat)
            yield=yield+dshayieldh(e2,emuthr,3,yieldk,istat)
          endif

c---------- z0 gamma channel ----------
          if (ch.eq.14) then
            yield=yield+0.5d0*dshayieldf(e1,emuthr,6,yieldk,istat)
          endif

          dshayield=yield

        else   ! not energetically allowed channel
          dshayield=0.0d0
          istat=mod(istat,2)+2
c          if (prtlevel.gt.0) then
            write(6,5000) 'error in dshayield: channel ',ch,' is not',
     +        ' energetically allowed.'
            write(6,*) 'model: ',idtag
c          endif
        endif

      endif

      if (haerr.gt.0.and.prtlevel.gt.0) then
        write(*,*) 'warning in dshayield for model ',idtag,
     &    ', yield ',yieldk,' and channel ',ch
        write(*,*) '  the integration over higgs decay angles ran',
     &    'into numerical problems.'
        write(*,*) '  the results can only be trusted as a lower',
     &    'bound.'
        istat=ibset(istat,2)
      endif

 5000 format(' ',a,i2,a,a)

      end




