*****************************************************************************
*** function yield calculates the yield above threshold (flxk=1,2 or 3)
*** or the differential yield (flxk=101, 102 or 103) from a given
*** annihilation channel. channels ch=1-14 are supported.
*** Note. Gluons (channel 12) are not included yet, this channel
*** returns a zero yield. Channel 13 (mu+ mu-) never yields anything,
*** but is included for compatibility with the halo annihilation routines.
*** if flxk = 1 or 101 - neutrino yields are given
***           2 or 102 - muon distributions at creation are given
***           3 or 103 - muon yields at the detector are given
*** the units are 1e-30 m**-2 (annihilation)**-1 for 1 and 3, and
*** 1e-30 m**-3 (annihilation)**-1 for 2.
*** For the differential yields, the units are the same plus
*** gev**-1 degree**-1.
***
*** author: joakim edsjo (edsjo@physics.berkeley.edu)
*** date: 1995
*** modified: dec 03, 1997
*****************************************************************************

      real*8 function dsmuyield(mneu,emuthr,thmax,ch,wh,flxk,istat)
      implicit none
      include 'dsmucom.h'
      include 'dsprep.h'
      include 'dsidtag.h'
      include 'dsio.h'

c------------------------ variables ------------------------------------

      real*8 mneu,emuthr,thmax
      real*8 mp1,mp2,e1,e2,flx
      integer ch,istat,flxk,fltyp
      character*2 wh

      logical chok

      logical first
      data first /.true./
      save first

c------------------------ functions ------------------------------------

      real*8 dsmuyieldf,dsmuyieldh

c-----------------------------------------------------------------------

      istat=0
      mferr=0
      mp1=0.d0
      mp2=0.d0
      if (flxk.lt.100) then
        fltyp=1      ! integrated yields
      else
        fltyp=2      ! differential yields
      endif

c--------------------------------------- if first call, load muon tables

      if (first) then
        flxtype(1)=0
        flxtype(2)=0
        first=.false.
        call dsmuinit(flxk)
      endif

      if ((flxk.ne.flxtype(1)).and.(flxk.ne.flxtype(2)))
     &  then
        if ((flxk.lt.100.and.flxtype(1).ne.0).or.
     &    (flxk.gt.100.and.flxtype(2).ne.0)) then
          write(*,*) 
     &     'warning in dsmuyield: you have requested a different',
     &     ' kind of yield than loaded. reloading necessary.'
        endif
        call dsmuinit(flxk)
      endif

c-----------------------------------------------------------------------
      if (ch.le.6) then              ! "fundamental" channel
        chok = .false.
        if (mneu.ge.map(ch)) chok = .true.
        if ((ch.eq.3.or.ch.eq.5.or.ch.eq.6).and.
     &     mneu.gt.(0.99*map(ch))) chok = .true.
        if (chok) then
          dsmuyield=dsmuyieldf(mneu,emuthr,thmax,ch,wh,fltyp,istat)
        else
          dsmuyield=0.0
          istat=mod(istat,2)+2
            write(6,5000) 'error in dsmuyield: channel ',ch,' is not',
     &        ' energetically allowed.'
            write(6,*) 'model: ',idtag
        endif

      else                           ! "complex" channel

c...determine masses of the annihilation particles
        if (ch.eq.7) then
          mp1=mh(1)         ! h10 mass
          mp2=mh(3)         ! h30 mass
        elseif (ch.eq.8) then
          mp1=map(6)        ! z0 mass
          mp2=mh(1)         ! h10 mass
        elseif (ch.eq.9) then
          mp1=map(6)        ! z0 mass
          mp2=mh(2)         ! h20 mass
        elseif (ch.eq.10) then
          mp1=map(5)        ! w+- mass
          mp2=mh(4)         ! h+- mass
        elseif (ch.eq.11) then
          mp1=mh(2)         ! h20 mass
          mp2=mh(3)         ! h30 mass
        elseif (ch.eq.14) then
          mp1=map(6)        ! z0 mass
          mp2=0.0d0         ! gamma mass
        else                ! not a supported channel
          dsmuyield=0.0d0
          return
        endif

c...if energetically allowed channel, go on...
        if (mneu.ge.0.995d0*((mp1+mp2)/2.0d0)) then

c...calculate the energy of the annihilation particles
          e1=((2.0d0*mneu)**2-mp2**2+mp1**2)/(4.0d0*mneu)
          e2=2.0d0*mneu-e1
          e1=max(e1,mp1+0.001d0)
          e2=max(e2,mp2+0.001d0)

c...check different annihilation channels
          flx=0.0d0

c---------- h10 h30 channel ----------
          if (ch.eq.7) then
            flx=flx+dsmuyieldh(e1,emuthr,thmax,1,wh,fltyp,istat)
            flx=flx+dsmuyieldh(e2,emuthr,thmax,3,wh,fltyp,istat)
          endif

c---------- z0 h10 channel ----------
          if (ch.eq.8) then
            flx=flx+0.5d0*dsmuyieldf(e1,emuthr,thmax,6,wh,fltyp,istat)
            flx=flx+dsmuyieldh(e2,emuthr,thmax,1,wh,fltyp,istat)
          endif

c---------- z0 h20 channel ----------
          if (ch.eq.9) then
            flx=flx+0.5d0*dsmuyieldf(e1,emuthr,thmax,6,wh,fltyp,istat)
            flx=flx+dsmuyieldh(e2,emuthr,thmax,2,wh,fltyp,istat)
          endif

c---------- w+h- w-h+ channel ----------
c...this calculation gives a mean of the two channels w+h- & w-h+
          if (ch.eq.10) then
            flx=flx+0.5d0*dsmuyieldf(e1,emuthr,thmax,5,wh,fltyp,istat)
            flx=flx+dsmuyieldh(e2,emuthr,thmax,4,wh,fltyp,istat)
          endif

c---------- h20 h30 channel ----------
          if (ch.eq.11) then
            flx=flx+dsmuyieldh(e1,emuthr,thmax,2,wh,fltyp,istat)
            flx=flx+dsmuyieldh(e2,emuthr,thmax,3,wh,fltyp,istat)
          endif

c---------- z0 gamma channel ----------
          if (ch.eq.14) then
            flx=flx+0.5d0*dsmuyieldf(e1,emuthr,thmax,6,wh,fltyp,istat)
          endif

          dsmuyield=flx

        else   ! not energetically allowed channel
          dsmuyield=0.0d0
          istat=mod(istat,2)+2
c          if (prtlevel.gt.0) then
            write(6,5000) 'error in dsmuyield: channel ',ch,' is not',
     +        ' energetically allowed.'
            write(6,*) 'model: ',idtag
c          endif
        endif

      endif

      if (mferr.gt.0) then
        write(*,*) 'warning in dsmuyield for model ',idtag,
     &    ', yield ',flxk,' and channel ',ch
        write(*,*) '  the integration over higgs decay angles ran',
     &    'into numerical problems.'
        write(*,*) '  the results can only be trusted as a lower',
     &    'bound.'
        mfistat=ibset(mfistat,2)
        istat=ibset(istat,2)
      endif

 5000 format(' ',a,i2,a,a)

      end




